use crate::spotify::{get_entry_kind, AlbumInfo};
use crate::TrackInfo;
use log::info;
use spotify::ContentKind;
use youtube::Video;

pub mod spotify;
mod youtube;

#[derive(Debug, Clone, PartialEq, Eq, Hash)]
pub(crate) struct TrackItem {
    pub(crate) spotify_track: Option<TrackInfo>,
    pub(crate) youtube_track: Option<Vec<Video>>,
}

#[derive(Debug, Clone, PartialEq, Eq, Hash)]
pub(crate) struct AlbumItem {
    pub(crate) spotify_track: Option<AlbumInfo>,
}

// The enum holds all the currently supported type of Id which the engine can search for
#[derive(Debug, Clone, PartialEq, Eq, Hash)]
pub(crate) enum ServiceIdKind {
    Spotify(String),
    Youtube(String),
    Automatic(String),
}

// This struct will allow us in the future to search, cache and store data and metadata regarding
// tracks, albums and playlists
#[derive(Debug, Clone)]
pub(crate) struct Engine {
    spotify: spotify::Client,
    youtube: youtube::Client,
}

impl Engine {
    pub(crate) async fn new() -> Self {
        Engine {
            spotify: spotify::Client::new().await,
            youtube: youtube::Client::new().await,
        }
    }

    pub(crate) fn new_with_dependencies(
        spotify_client: spotify::Client,
        youtube_client: youtube::Client,
    ) -> Self {
        Engine {
            spotify: spotify_client,
            youtube: youtube_client,
        }
    }

    pub(crate) async fn get_song_by_name(&self, name: &str) -> TrackItem {
        todo!("In the future it would be possible to search for all metadata on a record from this call")
    }

    pub(crate) async fn get_song_from_spotify_id(&self, message: &str) -> TrackItem {
        let entry_kind = spotify::get_entry_kind(message);
        let track_info = match entry_kind {
            Some(entry) => match entry {
                ContentKind::Track(id) => self.spotify.get_track(id.as_str()).await,
                _ => None,
            },
            None => None,
        };

        if track_info.is_some() {
            let ti = track_info.unwrap();
            let youtube_search = match self
                .youtube
                .search_video(
                    format!(
                        "{}{}",
                        ti.artists
                            .get(0)
                            .map(|artist| format!("{} - ", artist))
                            .unwrap_or("".to_string()),
                        ti.name
                    )
                    .as_str(),
                    None,
                )
                .await
            {
                Err(_) => None,
                Ok(search) => Some(search),
            };

            return TrackItem {
                spotify_track: Some(ti),
                youtube_track: youtube_search.map(|search| search.items),
            };
        }
        return TrackItem {
            spotify_track: None,
            youtube_track: None,
        };
    }

    pub(crate) async fn get_album_by_spotify_id(&self, message: &str) -> AlbumItem {
        let entry_kind = spotify::get_entry_kind(message);

        let album_info = match entry_kind {
            Some(entry) => match entry {
                ContentKind::Album(id) => self.spotify.get_album(id.as_str()).await,
                _ => None,
            },
            None => None,
        };

        AlbumItem {
            spotify_track: album_info,
        }
    }
}

pub(crate) fn get_spotify_kind(spotify_id: &str) -> Option<ContentKind> {
    get_entry_kind(spotify_id)
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn should_search_track_by_spotify_id() {
        todo!("Implement me!")
    }
}
