use crate::search::{AlbumItem, TrackItem};
use crate::TrackInfo;
use log::{info, log};
use std::borrow::Borrow;
use std::collections::HashSet;
use std::fmt::format;
use std::time::Duration;
use teloxide::repl;

pub mod utils;

static MAX_ARTISTS_CHARS: usize = 140;

pub(crate) fn format_track_message(track_info: TrackItem) -> Option<String> {
    // Let's avoid copying all the structure...we place it in the heap and pass the pointer to all the other functions
    let boxed_info = Box::new(track_info);
    if boxed_info.spotify_track.is_none() && boxed_info.youtube_track.is_none() {
        return None;
    }

    let mut reply = format!(
        "Track information:\n\
                                🎵 Track name: {}\n\
                                🧑‍🎤 Artist(s): {}\n\
                                ⏳ Duration: {}",
        get_track_name(boxed_info.clone()),
        get_track_artists(boxed_info.clone()),
        get_track_duration(boxed_info.clone())
    );

    let possible_first_video_link = boxed_info
        .youtube_track
        .and_then(|x| x.get(0).map(|x| format!("https://youtu.be/{}", x.video_id)));

    if possible_first_video_link.is_some() {
        reply.push_str(
            format!("\n\n🔗Youtube link: {}", possible_first_video_link.unwrap()).as_str(),
        );
    }

    Some(reply)
}

pub(crate) fn format_album_message(album_info: AlbumItem) -> Option<String> {
    let boxed_info = Box::new(album_info);
    if boxed_info.spotify_track.is_none() {
        return None;
    }

    let mut artists = get_album_artists(boxed_info.clone());
    let mut artists_names = "Unknown artist list".to_string();

    if artists.len() > 0 {
        artists_names = itertools::join(&artists, ", ");
    }

    let mut reply = format!(
        "Album information:\n\
                                    🎵 Album name: {}\n\
                                    🧑‍🎤 {} artist(s): {}",
        get_album_name(boxed_info.clone()),
        artists.len(),
        artists_names
    );

    let mut album_genres = get_album_genres(boxed_info.clone());
    if album_genres.len() > 0 {
        reply.push_str(format!("\n💿 Genre(s): {}", itertools::join(&artists, ", ")).as_str());
    }

    return Some(reply);
}

fn get_album_name(ai: Box<AlbumItem>) -> String {
    ai.spotify_track
        .map(|s| s.name)
        .unwrap_or("Unknown album name".to_string())
}

fn get_album_genres(ai: Box<AlbumItem>) -> HashSet<String> {
    if ai.spotify_track.is_some() {
        return ai.spotify_track.unwrap().genres.into_iter().collect();
    }

    return HashSet::new();
}

fn get_album_artists(ai: Box<AlbumItem>) -> HashSet<String> {
    if ai.spotify_track.is_some() {
        return ai.spotify_track.unwrap().artists.into_iter().collect();
    }

    return HashSet::new();
}

fn get_track_name(ti: Box<TrackItem>) -> String {
    if ti.spotify_track.is_some() {
        ti.spotify_track.unwrap().name
    } else {
        ti.youtube_track
            .and_then(|youtube_tracks| youtube_tracks.get(0).map(|t| t.title.clone()))
            .unwrap_or("Unknown title".to_string())
    }
}

fn get_track_artists(ti: Box<TrackItem>) -> String {
    if ti.spotify_track.is_some() {
        utils::truncate_with_dots(
            ti.spotify_track.unwrap().artists.join(", "),
            MAX_ARTISTS_CHARS,
        )
    } else {
        ti.youtube_track
            .and_then(|youtube_tracks| youtube_tracks.get(0).map(|t| t.author.clone()))
            .unwrap_or("Unknown artist".to_string())
    }
}

fn get_track_duration(ti: Box<TrackItem>) -> String {
    if ti.spotify_track.is_some() {
        utils::human_readable_duration(ti.spotify_track.unwrap().duration)
    } else {
        ti.youtube_track
            .map(|youtube_tracks| {
                let seconds_in_string =
                    youtube_tracks.get(0).map(|t| t.length_seconds).unwrap_or(0);

                utils::human_readable_duration(Duration::from_secs(seconds_in_string))
            })
            .unwrap_or("Unknown duration".to_string())
    }
}
